#!/bin/bash
PROJECT_ROOT='/root/monitoring_alert_checkers'
. $PROJECT_ROOT/utils.sh

ENV_FILE='/etc/service_env'

[ ! -e "$ENV_FILE" ] && { write_log "Env file $ENV_FILE is missing" ; exit 1 ; }
. "$ENV_FILE"

PROJECT_ROOT=${PROJECT_ROOT:='/root/monitoring_alert_checkers'}
CRON_INTERVAL_MIN=${CRON_INTERVAL_MIN:=15}
LOG_FILE=${LOG_FILE:='managed_servers.log'}

if [ ! -d "$ENV_PATH" ]; then
  write_log "> Creating virtualenv in $ENV_PATH"
  /usr/bin/python3 -m venv "$ENV_PATH"
  write_log "+ Done!"
else
  write_log "+ Virtualenv at $ENV_PATH is already set up"
fi

write_log "> Installing dependencies"
. "$ENV_PATH"/bin/activate
pip install -r $PROJECT_ROOT/requirements.txt > /dev/null
write_log "+ Done!"


write_log "> Adding app log to logrotate"
cat <<- CONF > /etc/logrotate.d/"$LOG_FILE"
/root/$LOG_FILE {
    monthly
    maxsize 100M
    create 644 root root
    rotate 12
    missingok
}
CONF

write_log '> Checking cron job'
if cron="$(crontab -l 2>/dev/null | grep ${PROJECT_ROOT}/run.sh)"; then
  write_log "Cron job is already set up"
  echo "$cron"
else
  write_log "Setting up cron job"
  (crontab -l 2>/dev/null; echo "*/$CRON_INTERVAL_MIN * * * * /bin/bash ${PROJECT_ROOT}/run.sh") | crontab -
fi

write_log "> Checking configuration"
check_exit ex logrotate --debug /etc/logrotate.d/"$LOG_FILE"

write_log "+ App has been configured"
